####################################################################
#############################   PETPA   ############################
####################################################################
# compute 2nd hyperpolarizabilities of p-NA in vacuum
# compute 2nd hyperpolarizabilities of p-NA in the PIM environment
# Calculate the TPA cross-sections for both cases
# Compare the ratio to obtain the PETPA enhancement factor
####################################################################


######################### Initializations ##########################
import os, sys
import numpy as np
from scm.plams import *
init()


############################ Filenames #############################
geometryFileQM   = 'pNA.xyz'
geometryFileDIM  = 'Ag2057.xyz'
parameterFileDIM = 'Ag_jc'


######################### Unit Conversion ##########################
# conversion a.u. to GM i.e. to 10E-50 * cm^4 * s / photon
au2GM    = 1.896831653271693 # = 5.2918e-9**4 * 2.418884326505E-17 * 1.0E50
alphaf   = 0.0072973525664
factorGM = au2GM * 4 * np.pi**3 * alphaf**2 / 15


############################## Models ##############################
# Load structure of the QM region
molQM = Molecule(geometryFileQM)

# Generate the DIM region, load and shift one cluster, then rotate its copy and merge both parts into a single molecule instance
dimRegionL = Molecule(geometryFileDIM)
dimRegionL.translate((.0, .0, -4.-2.81396650))
dimRegionU = dimRegionL.copy()
dimRegionU.rotate([-1., .0, .0, .0, 1., .0, .0, .0, -1])
dimRegion = dimRegionL + dimRegionU


########################## DFT-Settings #############################
# QM Settings
setQM = Settings()
setQM.input.basis.type           = 'TZP'
setQM.input.basis.core           = 'None'
setQM.input.basis.createoutput   = 'None'
setQM.input.xc.model             = 'SAOP'
setQM.input.symmetry             = 'nosym'
setQM.input.allpoints            = ''
setQM.input.numericalquality     = 'good'

freq                             = 0.06284135 # = 1.71 eV
setQM.input.aoresponse.frequency = '3 '+str(freq)+' '+str(freq)+' '+str(-freq)+' Hartree'
setQM.input.aoresponse.scf       = 'iterations 50'
setQM.input.aoresponse.gamma     = ''
setQM.input.aoresponse.tpa       = ''
setQM.input.aoresponse.ALDA      = ''
setQM.input.aoresponse.lifetime  = '0.0037'

########################## PIM-Settings #############################
# DIM Settings, generates the blocks DIMQM and DIMPAR
setDIM = Settings()
setDIM.input.DIMQM.PIM        = ''
setDIM.input.DIMQM.ALGORITHM  = 'Direct'
setDIM.input.DIMQM.NITER      = '2000'
setDIM.input.DIMQM.FREQUENCY  = ''
setDIM.input.DIMQM.LOCALFIELD = ''

setDIM.input.DIMPAR['_1']     = 'Ag'
setDIM.input.DIMPAR['_2']     = 'rad = 1.4445'
setDIM.input.DIMPAR['_3']     = 'exp '+ os.path.join(os.path.dirname(os.path.abspath(__file__)), parameterFileDIM)
setDIM.input.DIMPAR['_4']     = 'SUBEND'
setDIM.input.DIMPAR['_5']     = 'XYZ'
setDIM.input.DIMPAR['_6']     = len(dimRegion)
for iAt, atom in enumerate(dimRegion):
   setDIM.input.DIMPAR['_'+str(7+iAt)] = atom.str()
setDIM.input.DIMPAR['_'+str(7+len(dimRegion))] = 'SUBEND'

################### Calculate TPA Cross Sections ####################
# Setup and run TPA job without DIM environment
jobTPA         = ADFJob(molecule=molQM, settings=setQM)
resultsTPA     = jobTPA.run()
gammaImagTPA   = np.array([float(elem.split()[-1]) for elem in resultsTPA.grep_output(pattern=' gamma                             real              imaginary', options='-A81')[1:]]).reshape((3,3,3,3))
sigmaTPA       = factorGM * freq**2 * sum(gammaImagTPA[i][i][j][j] + gammaImagTPA[i][j][j][i] + gammaImagTPA[i][j][i][j] for i in range(3) for j in range(3))

# Setup and run PETPA job with DIM environment
jobPETPA       = ADFJob(molecule=molQM, settings=setQM+setDIM)
resultsPETPA   = jobPETPA.run()
gammaImagPETPA = np.array([float(elem.split()[-1]) for elem in resultsPETPA.grep_output(pattern=' gamma                             real              imaginary', options='-A81')[1:]]).reshape((3,3,3,3))
sigmaPETPA     = factorGM * freq**2 * sum(gammaImagPETPA[i][i][j][j] + gammaImagPETPA[i][j][j][i] + gammaImagPETPA[i][j][i][j] for i in range(3) for j in range(3))

######################## Print Final Results ########################
print('omega             = ', freq * 27.2114)
print('gammaI(TPA)       = ', gammaImagTPA)
print('gammaI(PETPA)     = ', gammaImagPETPA)
print('sigma(TPA)        = ', sigmaTPA)
print('sigma(PETPA)      = ', sigmaPETPA)
print('PETPA enhancement = ', sigmaPETPA / sigmaTPA)


finish()
